//
//  OpenApi.h
//  OpenSdkTest
//
//  Created by aine sun on 3/15/12.
//  Copyright 2012 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#import "OpenSdkRequest.h"
#import "OpenSdkOauth.h"
#import "OpenSdkResponse.h"
#import "WBAuthorizeWebView.h"

@class OpenApi;
@protocol QQApiDelegate <NSObject>

@optional

// If you try to log in with logIn or logInUsingUserID method, and
// there is already some authorization info in the Keychain,
// this method will be invoked.
// You may or may not be allowed to continue your authorization,
// which depends on the value of isUserExclusive.
- (void)qqApiAlreadyLoggedIn:(OpenApi *)engine;

// Log in successfully.
- (void)qqApiDidLogIn:(OpenApi *)engine;

// Failed to log in.
// Possible reasons are:
// 1) Either username or password is wrong;
// 2) Your app has not been authorized by Sina yet.
- (void)qqApi:(OpenApi *)engine didFailToLogInWithError:(NSString *)error;

// Log out successfully.
- (void)qqApiDidLogOut:(OpenApi *)engine;

// When you use the WBEngine's request methods,
// you may receive the following four callbacks.
- (void)qqApiNotAuthorized:(OpenApi *)engine;
- (void)qqApiAuthorizeExpired:(OpenApi *)engine;

- (void)qqApi:(OpenApi *)engine requestDidFailWithError:(NSString *)error;
- (void)qqApi:(OpenApi *)engine requestDidSucceedWithResult:(id)result;

@end


@interface OpenApi : UIViewController<UINavigationControllerDelegate, UIImagePickerControllerDelegate,WBAuthorizeWebViewDelegate> {

    OpenSdkOauth *_OpenSdkOauth;
    OpenSdkRequest *_OpenSdkRequest;
    OpenSdkResponse *_OpenSdkResponse;
    
    NSMutableDictionary *_publishParams;
    NSString *_filePathName;
    uint16_t _retCode;
    
    id<QQApiDelegate> _qqApiDelegate;
}

@property (nonatomic,retain) NSString *filePathName;
@property (nonatomic) uint16_t retCode;
@property (nonatomic,assign) id<QQApiDelegate> delegate;


+ (OpenApi*)shareOpenApi;
/*
 * 初始化
 */
- (id)initForApi:(NSString*)appKey appSecret:(NSString*)appSecret accessToken:(NSString*)accessToken accessSecret:(NSString*)accessSecret openid:(NSString *)openid oauthType:(uint16_t)oauthType;
/*
 *登陆
 */
- (void)login;

- (void)logout;

/*
 * 发表微博
 */
- (void)publishWeibo:(NSString *)weiboContent jing:(NSString *)jing wei:(NSString *)wei format:(NSString *)format clientip:(NSString *)clientip syncflag:(NSString *)syncflag;

/*
 * 选择图片
 */
//- (void) insertImage:(id)delegate;

/*
 * 发表带图片微博
 */
- (void) publishWeiboWithImage:(NSString *)filePath weiboContent:(NSString *)weiboContent jing:(NSString *)jing wei:(NSString *)wei format:(NSString *)format clientip:(NSString *)clientip syncflag:(NSString *)syncflag;

/*
 * 获取用户信息
 */
- (NSString*) getUserInfo:(NSString *)format;

/*
 * 拉取我收听的人列表
 */
- (void) getMyIdollist:(NSString *)format reqnum:(NSString *)reqnum startIndex:(NSString *)startIndex install:(NSString *)install pageCount:(NSInteger)pageCount;

/*
 * 拉取我的收听列表
 */
- (void) getMyFanslist:(NSString *)format reqnum:(NSString *)reqnum startIndex:(NSString *)startIndex mode:(NSString *)mode install:(NSString *)install;

@end
