//
//  RenRenApi.h
//  RenRenSDK
//
//  Created by rannger on 12-8-24.
//  Copyright (c) 2012年 rannger. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "RenRenOAuth.h"
#import "ASIHTTPRequest.h"
#import "types.h"

@class RenRenApi;
@protocol RenRenApiDelegate <NSObject>
@optional
- (void)renRenApiDidLogin;

- (void)renRenapiDidLogout;

- (void)renRenApiNotAuthorized:(RenRenApi *)renRenApi;

- (void)renRenApi:(RenRenApi *)renRenApi requestDidFailWithError:(NSString *)error;

- (void)renRenApi:(RenRenApi *)renRenApi requestDidSucceedWithResult:(NSString*)result;
@end

@interface RenRenApi : NSObject <ASIHTTPRequestDelegate>
{
    RenRenOAuth* oauth_;
    NSMutableArray* requestArray_;
    id<RenRenApiDelegate> delegate_;
}
@property (nonatomic,retain) RenRenOAuth* oauth;
@property (nonatomic,assign) id<RenRenApiDelegate> delegate;
+ (RenRenApi*)shareRenRenApi;

- (void)logout;
/*
 *得到当前session的用户ID。
 *@scope：非必须参数。以空格分隔的权限列表，若不传递此参数，代表请求用户的默认权限。全部权限请参考http://wiki.dev.renren.com/wiki/%E6%9D%83%E9%99%90%E5%88%97%E8%A1%A8。
 */
- (void)loginWithUserName:(NSString*)userName
                 password:(NSString*)password
                    scope:(NSArray*)scope;
/*
 *产生web登陆的认证请求
 *默认情况下，会要求用户授予用于访问基本信息的权限，例如用户公共信息、好友关系等。如果应用需要的权限超过基本信息权限，
 *就需要要求用户授予深层次的权限。这个过程是通过添加人人OAuth 2.0 'scope'请求参数实现的：
 *scope：非必须参数。以空格分隔的权限列表，若不传递此参数，代表请求用户的默认权限。
 *       全部权限请参考http://wiki.dev.renren.com/wiki/%E6%9D%83%E9%99%90%E5%88%97%E8%A1%A8
 */
- (void)loginWithWebView:(UIWebView*)webView purview:(NSArray*)scope;



/**************************************获取用户信息信息***********************************************/
/*
 *得到用户的信息，支持批量获取。
 *fields string	返回的字段列表，可以指定返回那些字段，用逗号分隔。如：uid,name,sex,star,zidou,vip,birthday,tinyurl,
 *              headurl,mainurl,hometown_location work_history,university_history。
 *              如果不传递此参数默认返回uid,name,tinyurl,headhurl,zidou,star。
 *              此参数请指定为你需要的字段，为了提升响应速度，你不需要用到的字段请不要传入。
 *              注意：当不传入session_key或没有权限时，fields失效！
 */
- (void)getSelfInfo:(NSArray*)fields;
/*
 *得到当前登录用户的好友列表，得到的只是含有好友id的列表.
 *@recordPerPage 每页记录数
 *@pageNo 页码
 */
//- (void)getCurrentLoginFriendsWithRecordPerPage:(NSInteger)recordPerPage pageNo:(NSInteger)pageNo;

/**************************************发送状态信息**************************************************/
/*
 *用户更新状态
 *此API需要用户授予 status_update 权限(在OAuth2.0授权中由scope参数指定)。
 *@status 状态内容
 *@placeId 地点id(可为空)
 */
- (void)publishState:(NSString*)status inPlace:(NSString*)placeId;

/**************************************发送新鲜事信息************************************************/
/*
 *以当前会话的用户身份发送应用的自定义新鲜事。
 *@description 新鲜事内容
 *@name 新鲜事标题
 *@titleUrl 新鲜事标题链接URL(可为空）
 *@image 新鲜事图片链接URL(可为空）
 */
- (void)publishFeed:(NSString*)description
          feedTitle:(NSString*)name
           titleUrl:(NSString*)titleUrl
              image:(NSString*)imagePath;
/**************************************分享*********************************************************/
/*
 *发布人人网站外分享，例如分享视频、音乐、链接等
 *@linkUrl 要分享的站外链接url
 *@type 分享类型
 */
- (void)shareSitesOutsideContentWithLink:(NSString*)linkUrl
                                    type:(ShareType)type;

/*
 *泛用接口
 *@url 发起URL
 *@params 参数字典
 */
- (void)requestWithURL:(NSString*)url
                 param:(NSDictionary*)params;


@end
