//
//  RenRenApi.m
//  RenRenSDK
//
//  Created by rannger on 12-8-24.
//  Copyright (c) 2012年 rannger. All rights reserved.
//

#import "RenRenApi.h"
#import "RRSDKBase.h"
#import "JSON.h"
#import "RenRenRequest.h"

static RenRenApi* renRenApi=nil;
@implementation RenRenApi

@synthesize oauth=oauth_;
@synthesize delegate=delegate_;

- (id)init
{
    if (self=[super init]) {
        oauth_=[[RenRenOAuth alloc] init];
        requestArray_=[[NSMutableArray alloc] init];
    }
    return self;
}

- (void)dealloc
{
    [oauth_ release];
    [requestArray_ release];
    [super dealloc];
}

+ (RenRenApi*)shareRenRenApi
{
    if (renRenApi==nil) {
        renRenApi=[[RenRenApi alloc] init];
        [renRenApi.oauth readAuthorizeDataFromKeychain];
    }
    return renRenApi;
}

#pragma mark - 人人网开放平台API
#pragma mark -
#pragma mark 登陆／认证
- (void)loginWithUserName:(NSString*)userName
                 password:(NSString*)password
                    scope:(NSArray*)scope
{
    ASIHTTPRequest *request=[RenRenRequest loginWithUserName:userName password:password scope:scope];
    [requestArray_ addObject:request];
}

- (void)loginWithWebView:(UIWebView*)webView purview:(NSArray*)scope
{
    [webView loadRequest:[RenRenRequest authorizeRequest:scope]];
}


- (void)logout
{
    [self.oauth deleteAuthorizeDataInKeychain];
    if (delegate_&&[delegate_ respondsToSelector:@selector(renRenapiDidLogout)]) {
        [delegate_ renRenapiDidLogout];
    }
}

#pragma mark 用户信息
- (void)getSelfInfo:(NSArray*)fields
{
    ASIHTTPRequest *request=[RenRenRequest getUserInfoUserIds:nil field:fields];
    [requestArray_ addObject:request];
}

#pragma mark - 更新状态
- (void)publishState:(NSString*)status inPlace:(NSString*)placeId
{
    ASIHTTPRequest *request=[RenRenRequest publishState:status inPlace:placeId];
    [requestArray_ addObject:request];
}

#pragma mark - 新鲜事
- (void)publishFeed:(NSString*)feed
          feedTitle:(NSString*)title
           titleUrl:(NSString*)titleUrl
              image:(NSString*)imagePath
{
    
    ASIHTTPRequest *request=[RenRenRequest publishFeed:feed
                                             feedTitle:title
                                              titleUrl:titleUrl
                                        optionalParams:imagePath?[NSDictionary dictionaryWithObject:imagePath forKey:@"image"]:nil];
    [requestArray_ addObject:request];
}

#pragma mark - 分享
- (void)shareSitesOutsideContentWithLink:(NSString*)linkUrl
                                    type:(ShareType)type
{
    ASIHTTPRequest *request=[RenRenRequest shareSitesOutsideContentWithLink:linkUrl type:type];
    [requestArray_ addObject:request];
}
#pragma mark - 泛用接口
- (void)requestWithURL:(NSString*)url
                 param:(NSDictionary*)params
{
    ASIHTTPRequest *request=[RenRenRequest requestWithURL:url param:params method:@"POST" delegate:self];
    [requestArray_ addObject:request];
}

#pragma mark - ASIHTTPRequestDelegate

- (void)requestFinished:(ASIHTTPRequest *)request
{
    NSLog(@"%@",[request responseString]);
    NSDictionary* dic=[[request responseString] JSONValue];
    
    if (nil==self.oauth.accessToken&&[[dic allKeys] containsObject:@"access_token"]) {
        self.oauth.accessToken=[dic objectForKey:@"access_token"];
        self.oauth.expiresTime=[[dic objectForKey:@"expires_in"] intValue];
        self.oauth.scopes=[[dic objectForKey:@"scope"] componentsSeparatedByString:@" "];
        if (delegate_&&[delegate_ respondsToSelector:@selector(renRenApiDidLogin)])
            [delegate_ renRenApiDidLogin];
        [self.oauth saveAuthorizeDataToKeychain];
    }
    else if([[dic allKeys] containsObject:@"error_code"])
    {
        NSArray* errorCode=[NSArray arrayWithObjects:@"450",@"452",@"453",@"2000",@"2001",@"2002", nil];
        NSString* errStr=[NSString stringWithFormat:@"%@",[dic objectForKey:@"error_code"]];
        if ([errorCode containsObject:errStr]) {
            if (delegate_&&[delegate_ respondsToSelector:@selector(renRenApiNotAuthorized:)])
                [delegate_ renRenApiNotAuthorized:self];
        }
        else
        {
            if (delegate_&&[delegate_ respondsToSelector:@selector(renRenApi:
                                                                   requestDidFailWithError:)]) 
                [delegate_ renRenApi:self requestDidFailWithError:[dic objectForKey:@"error_code"]];
        }
    }
    else if (delegate_&&[delegate_ respondsToSelector:@selector(renRenApi:
                                                          requestDidSucceedWithResult:)])
    {
        [delegate_ renRenApi:self requestDidSucceedWithResult:[request responseString]];
    }
    [requestArray_ removeObject:request];
}

- (void)requestFailed:(ASIHTTPRequest *)request
{
     NSLog(@"%@",[request responseString]);
    [delegate_ renRenApi:self requestDidFailWithError:[request responseString]];
    [requestArray_ removeObject:request];
}


@end
