//
//  RenRenApi.m
//  RenRenSDK
//
//  Created by rannger on 12-8-24.
//  Copyright (c) 2012年 rannger. All rights reserved.
//

#import "RenRenOAuth.h"
#import "RenRenRequest.h"
#import "RRSDKBase.h"
#import "SFHFKeychainUtils.h"

#define kRenrenURLSchemePrefix @"RenRen_"
#define kRenRenKeychainServiceNameSuffix    @"_RenRenServiceName"
#define kRenRenKeychainScopes               @"RenRenScopes"
#define kRenRenKeychainAccessToken          @"RenRenAccessToken"
#define kRenRenKeychainExpireTime           @"RenRenExpireTime"

@interface RenRenOAuth ()

- (NSString *)urlSchemeString;

@end

@implementation RenRenOAuth
@synthesize accessToken=accessToken_;
@synthesize expiresTime=expiresTime_;
@synthesize scopes=scopes_;

- (void)dealloc
{
    [accessToken_ release];
    [scopes_ release];
    [super dealloc];
}

- (NSString *)urlSchemeString
{
    return [NSString stringWithFormat:@"%@%@", kRenrenURLSchemePrefix, [RRSDKBase renrenAppKey]];
}

- (void)saveAuthorizeDataToKeychain
{
    NSString *serviceName = [[self urlSchemeString] stringByAppendingString:kRenRenKeychainServiceNameSuffix];
    
    [SFHFKeychainUtils storeUsername:kRenRenKeychainScopes andPassword:[scopes_ componentsJoinedByString:@","] forServiceName:serviceName updateExisting:YES error:nil];
	[SFHFKeychainUtils storeUsername:kRenRenKeychainAccessToken andPassword:accessToken_ forServiceName:serviceName updateExisting:YES error:nil];
	[SFHFKeychainUtils storeUsername:kRenRenKeychainExpireTime andPassword:[NSString stringWithFormat:@"%d", expiresTime_] forServiceName:serviceName updateExisting:YES error:nil];
}

- (void)readAuthorizeDataFromKeychain
{
    NSString *serviceName = [[self urlSchemeString] stringByAppendingString:kRenRenKeychainServiceNameSuffix];
    self.scopes = [[SFHFKeychainUtils getPasswordForUsername:kRenRenKeychainScopes andServiceName:serviceName error:nil] componentsSeparatedByString:@","];
    self.accessToken = [SFHFKeychainUtils getPasswordForUsername:kRenRenKeychainAccessToken andServiceName:serviceName error:nil];
    self.expiresTime = [[SFHFKeychainUtils getPasswordForUsername:kRenRenKeychainExpireTime andServiceName:serviceName error:nil] intValue];
}

- (void)deleteAuthorizeDataInKeychain
{
//    self.userID = nil;
    self.accessToken = nil;
    self.expiresTime = 0;
    
    NSString *serviceName = [[self urlSchemeString] stringByAppendingString:kRenRenKeychainServiceNameSuffix];
    [SFHFKeychainUtils deleteItemForUsername:kRenRenKeychainScopes andServiceName:serviceName error:nil];
	[SFHFKeychainUtils deleteItemForUsername:kRenRenKeychainAccessToken andServiceName:serviceName error:nil];
	[SFHFKeychainUtils deleteItemForUsername:kRenRenKeychainExpireTime andServiceName:serviceName error:nil];
}


@end
