//
//  RenRenRequest.h
//  RenRenSDK
//
//  Created by rannger on 12-8-24.
//  Copyright (c) 2012年 rannger. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "ASIHTTPRequest.h"
#import "types.h"


@interface RenRenRequest : NSObject
/*
 *得到当前session的用户ID。
 *@scope：非必须参数。以空格分隔的权限列表，若不传递此参数，代表请求用户的默认权限。全部权限请参考http://wiki.dev.renren.com/wiki/%E6%9D%83%E9%99%90%E5%88%97%E8%A1%A8。
 */
+ (ASIHTTPRequest*)loginWithUserName:(NSString*)userName
                            password:(NSString*)password
                               scope:(NSArray*)scope;
/*
 *产生web登陆的认证请求
 *默认情况下，会要求用户授予用于访问基本信息的权限，例如用户公共信息、好友关系等。如果应用需要的权限超过基本信息权限，
 *就需要要求用户授予深层次的权限。这个过程是通过添加人人OAuth 2.0 'scope'请求参数实现的：
 *scope：非必须参数。以空格分隔的权限列表，若不传递此参数，代表请求用户的默认权限。
 *       全部权限请参考http://wiki.dev.renren.com/wiki/%E6%9D%83%E9%99%90%E5%88%97%E8%A1%A8
 */
+ (NSURLRequest*)authorizeRequest:(NSArray*)scopes;
/*
 *得到用户的信息，支持批量获取。
 *uids	string	需要查询的用户的ID，多个ID用逗号隔开。例如：uids=1232,342,12324。
 *              建议都传入此参数。当此参数为空时，缺省值为session_key或access_token对应用户的ID。
 *fields string	返回的字段列表，可以指定返回那些字段，用逗号分隔。如：uid,name,sex,star,zidou,vip,birthday,tinyurl,
 *              headurl,mainurl,hometown_location work_history,university_history。
 *              如果不传递此参数默认返回uid,name,tinyurl,headhurl,zidou,star。
 *              此参数请指定为你需要的字段，为了提升响应速度，你不需要用到的字段请不要传入。
 *              注意：当不传入session_key或没有权限时，fields失效！
 */
+ (ASIHTTPRequest*)getUserInfoUserIds:(NSArray*)ids field:(NSArray*)fields;
/*
 *得到当前session的用户ID。
 */
+ (ASIHTTPRequest*)getLoggedInUserId;
/*
 *根据用户的id，以及相应在人人网的操作权限(接收email,更新状态等),来判断用户是否可以进行此操作。
 */
+ (ASIHTTPRequest*)hasAppPermission:(NSString*)permission userId:(NSInteger)uid;
/*
 *获取用户个人主页的信息，包括日志数、好友数、相册数、个人基本资料等
 * uid	    string	个人主页用户的ID
 * fields	string	需要获取的信息的字段列表，各个字段用逗号隔开。目前支持以下字段：base_info
 *                  （基本信息）,status（当前状态）,visitors_count（来访总数）,blogs_count（日志总数）,albums_count
 *                 （相册总数）,friends_count（好友总数）,guestbook_count（留言的总数）, status_count（状态的总数），
 *                  各个字段用逗号隔开。其中base_info包括性别、生日、家乡、网络。缺省或者用户有隐私设置时只返回用户ID，姓名，
 *                  头像，是否为星级用户。。
 */
+ (ASIHTTPRequest*)getProfileInfo:(NSInteger)uid fields:(NSArray*)fields;
/*
 *判断用户是否已经对应用授权。
 */
+ (ASIHTTPRequest*)isAppUser;

/*
 *得到当前登录用户的好友列表，得到的只是含有好友id的列表.
 */
//+ (ASIHTTPRequest*)getCurrentLoginFriendsWithRecordPerPage:(NSInteger)recordPerPage pageNo:(NSInteger)pageNo;
/*
 *用户更新状态
 *此API需要用户授予 status_update 权限(在OAuth2.0授权中由scope参数指定)。
 */
+ (ASIHTTPRequest*)publishState:(NSString*)status inPlace:(NSString*)placeId;
/*
 *以当前会话的用户身份发送应用的自定义新鲜事。
 *具体参数含义请点击http://wiki.dev.renren.com/wiki/Feed.publishFeed
 *fuck these optional parameters!
 */
+ (ASIHTTPRequest*)publishFeed:(NSString*)description
                     feedTitle:(NSString*)name
                      titleUrl:(NSString*)titleUrl
                optionalParams:(NSDictionary*)param;
/*
 *发布人人网站外分享，例如分享视频、音乐、链接等
 */
+ (ASIHTTPRequest*)shareSitesOutsideContentWithLink:(NSString*)linkUrl
                                               type:(ShareType)type;
/*
 *泛用接口
 *@url 发起URL
 *@params 参数字典
 *@method 发起方法,GET或POST
 *@delegate 异步调用结束代理
 */
+ (ASIHTTPRequest*)requestWithURL:(NSString*)url
                            param:(NSDictionary*)params
                           method:(NSString*)method
                         delegate:(id<ASIHTTPRequestDelegate>)delegate;
@end
