//
//  RenRenRequest.m
//  RenRenSDK
//
//  Created by rannger on 12-8-24.
//  Copyright (c) 2012年 rannger. All rights reserved.
//

#import "RenRenRequest.h"
#import "ASIFormDataRequest.h"
#import "RRSDKBase.h"
#import "RequestUrl.h"
#import "RenRenApi.h"

#define GET @"GET"
#define POST @"POST"

@interface RenRenRequest (private)
+ (ASIHTTPRequest*)requestWithParam:(NSDictionary*)params
                             method:(NSString*)method
                           delegate:(id<ASIHTTPRequestDelegate>)delegate;
+(ASIHTTPRequest*)requestWithURL:(NSURL*)URL
                           param:(NSString*)param
                        delegate:(id<ASIHTTPRequestDelegate>)delegate;
+ (NSURLRequest*)requestBaseUrl:(NSString*)baseUrl
                          param:(NSDictionary*)params;
+ (NSMutableDictionary*)generalParamDic;

+ (ASIHTTPRequest*)requestWithParam:(NSDictionary*)params;
@end

@implementation RenRenRequest

#pragma mark -
#pragma mark private
+(ASIHTTPRequest*)requestWithURL:(NSURL*)URL param:(NSString*)param delegate:(id<ASIHTTPRequestDelegate>)delegate
{
	if([param length]>0)
	{
		NSString* url=[URL absoluteString];
		url=[NSString stringWithFormat:@"%@?%@",url,param];
		NSLog(@"request url:%@",url);
        
        URL=[NSURL URLWithString:url];
        if(!URL)
        {
            //汉字时，生成URL为nil
            NSString* strUrl = [url stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
            URL=[NSURL URLWithString:strUrl];
        }
	}
	
	ASIHTTPRequest* request=[ASIHTTPRequest requestWithURL:URL];
	request.delegate=delegate;
	
	return request;
}

+ (ASIHTTPRequest*)requestWithParam:(NSDictionary*)params
{
    [params setValue:[RRSDKBase genSignature:params] forKey:@"sig"];
    return [self requestWithURL:RenRenBaseUrl param:params method:POST delegate:[RenRenApi shareRenRenApi]];
}

+ (ASIHTTPRequest*)requestWithURL:(NSString*)url
                            param:(NSDictionary*)params
                       method:(NSString*)method
                     delegate:(id<ASIHTTPRequestDelegate>)delegate
{
    
    if ([method isEqualToString:GET])
    {
        NSMutableArray* tempArr=[[NSMutableArray alloc] init];
        NSArray* arr=[params allKeys];
        for (NSString* key in arr) {
            [tempArr addObject:[NSString stringWithFormat:@"%@=%@",key,[params objectForKey:key]]];
        }
        ASIHTTPRequest* request=[self requestWithURL:[NSURL URLWithString:url]
                                               param:[tempArr componentsJoinedByString:@","]
                                            delegate:delegate];
        [request startAsynchronous];
        [tempArr release];
        return request;
    }
    else if([method isEqualToString:POST])
    {
        ASIFormDataRequest* request=[ASIFormDataRequest requestWithURL:[NSURL URLWithString:url]];
        request.delegate=delegate;
        request.timeOutSeconds=30;
        
        NSArray* arr=[params allKeys];
        for (NSString* key in arr)
            [request addPostValue:[params objectForKey:key] forKey:key];
        
        [request startAsynchronous];
        return request;
    }
    return nil;
}

+ (NSMutableDictionary*)generalParamDic
{
    NSMutableDictionary* params=[[NSMutableDictionary alloc] init];
    RenRenApi* api=[RenRenApi shareRenRenApi];
    [params setValue:@"1.0" forKey:@"v"];
    [params setValue:[RRSDKBase renrenAppKey] forKey:@"api_key"];
    [params setValue:api.oauth.accessToken forKey:@"access_token"];
    [params setValue:@"JSON" forKey:@"format"];
    return [params autorelease];
}

+ (NSURLRequest*)requestBaseUrl:(NSString*)baseUrl
                          param:(NSDictionary*)params
{
    NSMutableArray* tempArr=[[NSMutableArray alloc] init];
    NSArray* arr=[params allKeys];
    for (NSString* key in arr) {
        [tempArr addObject:[NSString stringWithFormat:@"%@=%@",key,[params objectForKey:key]]];
    }
    NSString* param=[tempArr componentsJoinedByString:@"&"];
    [tempArr release];
    return [NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@?%@",baseUrl,param]]];
}

+ (ASIHTTPRequest*)loginWithUserName:(NSString*)userName password:(NSString*)password scope:(NSArray*)scope
{
    NSDictionary* params=[NSDictionary dictionaryWithObjectsAndKeys:
                          @"password",@"grant_type",
                          userName,@"userName",
                          password,@"password",
                          [RRSDKBase renrenAppKey],@"client_id",
                          [RRSDKBase renrenSecretKey],@"client_secret",
                          [scope componentsJoinedByString:@"+"],@"scope", nil];
    
    return [self requestWithURL:AppAuthorizeURL param:params method:GET delegate:[RenRenApi shareRenRenApi]];
}

#pragma mark - 用户信息

+ (ASIHTTPRequest*)getUserInfoUserIds:(NSArray*)ids field:(NSArray*)fields
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"users.getInfo" forKey:@"method"];
    if (ids&&[ids count]!=0) {
        NSString* uids=[ids componentsJoinedByString:@","];
        [params setValue:uids forKey:@"uids"];
    }
    if (fields&&[fields count]!=0) {
        NSString* fieldstr=[fields componentsJoinedByString:@","];
        [params setValue:fieldstr forKey:@"fields"];
    }
    
    return [self requestWithParam:params];
}

+ (ASIHTTPRequest*)getLoggedInUserId
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"users.getLoggedInUser" forKey:@"method"];
    return [self requestWithParam:params];
}

+ (ASIHTTPRequest*)isAppUser
{
     NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"users.isAppUser" forKey:@"method"];
    return [self requestWithParam:params];
}


+ (ASIHTTPRequest*)getProfileInfo:(NSInteger)uid fields:(NSArray*)fields
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"users.getProfileInfo" forKey:@"method"];
    [params setValue:[NSString stringWithFormat:@"%d",uid] forKey:@"uid"];
    if (fields&&[fields count]!=0) {
        [params setValue:[fields componentsJoinedByString:@","] forKey:@"fields"];
    }
    return [self requestWithParam:params];
}

//+ (ASIHTTPRequest*)getVisitorsWithRecordPerPage:(NSInteger)recordPerPage pageNo:(NSInteger)pageNo
//{
//    NSMutableDictionary* params=[self generalParamDic];
//    [params setValue:@"users.getVisitors" forKey:@"method"];
//    [params setValue:[NSString stringWithFormat:@"%d",pageNo] forKey:@"page"];
//    [params setValue:[NSString stringWithFormat:@"%d",recordPerPage] forKey:@"count"];
//    return [self requestWithParam:params];
//}

+ (ASIHTTPRequest*)hasAppPermission:(NSString*)permission userId:(NSInteger)uid
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"users.hasAppPermission" forKey:@"method"];
    [params setValue:permission forKey:@"ext_perm"];
    [params setValue:[NSString stringWithFormat:@"%d",uid] forKey:@"uid"];
    return [self requestWithParam:params];
}

//+ (ASIHTTPRequest*)getCurrentLoginFriendsWithRecordPerPage:(NSInteger)recordPerPage pageNo:(NSInteger)pageNo
//{
//    NSMutableDictionary* params=[self generalParamDic];
//    [params setValue:@"friends.get" forKey:@"method"];
//    [params setValue:[NSString stringWithFormat:@"%d",pageNo] forKey:@"page"];
//    [params setValue:[NSString stringWithFormat:@"%d",recordPerPage] forKey:@"count"];
//    return [self requestWithParam:params];
//}

+ (ASIHTTPRequest*)publishState:(NSString*)status inPlace:(NSString*)placeId
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"status.set" forKey:@"method"];
    [params setValue:status forKey:@"status"];
    if (placeId&&[placeId length]!=0) {
        [params setValue:placeId forKey:@"place_id"];
    }
    return [self requestWithParam:params];
}

+ (ASIHTTPRequest*)publishFeed:(NSString*)description
                     feedTitle:(NSString*)name
                      titleUrl:(NSString*)titleUrl
                optionalParams:(NSDictionary*)param;
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"feed.publishFeed" forKey:@"method"];
    [params setValue:name forKey:@"name"];
    [params setValue:description forKey:@"description"];
    [params setValue:titleUrl forKey:@"url"];
    if (params&&[[params allKeys] count]!=0) {
        NSArray* keys=[params allKeys];
        for (NSString* key in keys) {
            [params setValue:[params valueForKey:key] forKey:key];
        }
    }
    return [self requestWithParam:params];
}

+ (ASIHTTPRequest*)shareSitesOutsideContentWithLink:(NSString*)linkUrl type:(ShareType)type
{
    NSMutableDictionary* params=[self generalParamDic];
    [params setValue:@"share.share" forKey:@"method"];
    [params setValue:linkUrl forKey:@"url"];
    [params setValue:[NSString stringWithFormat:@"%d",(NSInteger)type] forKey:@"type"];
    return [self requestWithParam:params];
}

#pragma mark - 用户认证／登陆
+ (NSURLRequest*)authorizeRequest:(NSArray*)scopes
{
    NSMutableDictionary *params=[[NSMutableDictionary alloc] init];
    [params setValue:[RRSDKBase renrenAppKey] forKey:@"client_id"];
    [params setValue:[RRSDKBase redirectUri] forKey:@"redirect_uri"];
    [params setValue:@"token" forKey:@"response_type"];
    
    if (scopes&&[scopes count]) {
        [params setValue:[scopes componentsJoinedByString:@","] forKey:@"scope"];
    }
    NSURLRequest * retval=[self requestBaseUrl:AuthorizeURL param:params];
    [params release];
    return retval;
}


@end
